/**
 * @(#)tupdate.cpp 2001/03/03
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

/**
 * This is a program that update the TSID tree to be in synch with HVSC upgrade
 * It uses a custom update file (updatexxx.thvs) generated with a script from
 * updatexx.hvs and corrected by hand.
 */

#include "tsid.h"
#include "sidt.h"
#include "osdepend.h"

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <dirent.h>

#define strequ(s1, s2) (strcmp((s1),(s2)) == 0)

//#define DEBUG_TUPDATE

// show debug string if DEBUG is defined
#ifdef DEBUG_TUPDATE
  #define TDEBUG(a) \
    cerr << "[TUPDATE] " << a;
#else
  #define TDEBUG(a) ;
#endif

// paramenter field
const int P_NOTHING   = 0x000;  // nothing passed
const int P_HELP      = 0x001;  // help syntax
const int P_INIT      = 0x002;  // init tree
const int P_UPDATE    = 0x004;  // update tree
const int P_BACKUP    = 0x008;  // backup tree
const int P_ERROR     = 0x010;  // error

const int E_OK      = 0;  // no error
const int E_MEMORY  = 1;  // no memory error
const int E_UNREAD  = 2;  // unread directory error
const int E_CHECK   = 3;  // error cheching directory
const int E_ALREADY = 4;  // already initialized
const int E_SYSTEM  = 5;  // error in system external call
const int E_WHO     = 6;  // error in find user
const int E_PARAM   = 7;  // error in paramenter passed
const int E_ENVIR   = 8;  // error in enviroment variable
const int E_MISSING = 9;  // missing file
const int E_NOHVSC  = 10; // no hvsc file found
const int E_REMOVE  = 11; // error removing directory
const int E_FROM    = 12; // missing FROM instruction
const int E_SYNTAX  = 13; // syntax error in parsing
const int E_INIT    = 14; // tree not initialized error
const int E_TO      = 15; // missing TO instructions
const int E_VERSION = 16; // invalid version
const int E_VER     = 17; // missing VER instruction

// last error in program
int lastError=E_OK; // no error

// path to the documentation directory
char *pathToDoc=NULL;

// version of the HVSC related with the TSID tree
char *hvscVer=NULL;

// version of the update apply to tsid tree
char tupdateVer;

// home directory
char *homeDir=NULL;

// tsid tree directory
char *tsidDir=NULL;

// name of tsid directory
char *TDir=".tsid_hvsc";

// name of documentation directory
char *TDoc="DOCUMENTS";

// name of the thvs update file
char *updateFile="update.thvs";

TSID tsid;

/**
 * Instruction DELETE file
 *
 * @param f the file in absolute path
 */
void instDELETE(char *f) {
  TDEBUG("DELETE " << f << endl);

  if (remove(f)!=0) {
    cerr << "Error removing the file, or file not present\n";
  }
}

/**
 * Instruction MOVE file1 to file2
 * If file2 esists, file1 is merged. File1 is then deleted.
 *
 * @param f1 the file to move abs path
 * @param f2 the file where to move abs path
 */
void instMOVE(char *f1, char *f2) {
  TDEBUG("MOVE " << f1 << " " << f2 << endl);

  SIDT sid1(f1);
  SIDT sid2(f2);

  unsigned int song;
  unsigned long second;
  unsigned long number;

  if (!sid1.read()) {       // read the first file
    cerr << "Error reading one file\n";
  }

  sid1.initForGet();
  while (sid1.getNext(&song, &second, &number)) {
    sid2.add(song, second, number);
  }

  if (!sid2.read()) {
    cerr << "Error reading one file\n";
  }
  if (!sid2.write()) {
    cerr << "Error writing the file\n";
  }

  //remove file1
  instDELETE(f1);
}

/**
 * Instruction SHIFT file, step, from song
 *
 * @param f the file in abs path
 * @param step the step to shift
 * @param from the song from where shifting
 */
void instSHIFT(char *f, int step, int from) {
  TDEBUG("SHIFT " << f << " " << step << " #" << from << endl);

  SIDT sid(f);

  if (!sid.read()) {
    cerr << "Error reading one file\n";
  }
  sid.shift(step, from);
  if (!sid.write()) {
    cerr << "Error writing the file\n";
  }
}

/**
 * Instruction SWAP tune n with tune m
 *
 * @param f the file to swap
 * @param n tune to swap with m
 * @param m tune to swap with n
 */
void instSWAP(char *f, int n, int m) {
  TDEBUG("SWAP " << f << " #" << n << " #" << m << endl);

  SIDT sid(f);

  if (!sid.read()) {
    cerr << "Error reading the file\n";
  }
  sid.swap(n,m);

  if (!sid.write()) {
    cerr << "Error writing the file\n";
  }
}

/**
 * Instruction GOES file1 tune n to file2 tune m
 * File1 is erased if it become empty.
 */
void instGOES(char *f1, char *f2, int n, int m) {
  TDEBUG("GOES " << f1 << " " << f2 << " #" << n << " #" << m << endl);

  SIDT sid1(f1);
  SIDT sid2(f2);

  unsigned int song;
  unsigned long second;
  unsigned long number;

  if (!sid1.read()) {
    cerr << "Error reading the file\n";
  }

  //search for the tune to move
  sid1.initForGet();
  while (sid1.getNext(&song, &second, &number)) {
    if (song==n) {
      sid1.remove(n);                             // remove the tune n
      sid2.add(m, second, number);                // add to tune m
      break;
    }
  }
  sid1.write();
  sid2.read();
  sid2.write();
}


/**
 * Get the version of the tree by try to read it from the directory
 * The readed information are stored in global directory.
 *
 * @param dir the directory that contains the version information
 * @return true if information are presents
 */
bool getTreeVer(char *dir) {
  dirent *dp;
  struct stat st;
  DIR    *dirp;
  int len;
  char *buf;
  bool present=false;   // true if information is present

  TDEBUG("Get TSID tree version\n");

  dirp=opendir(dir);

  if (dirp==NULL) {
    cerr << dir << " unreadable\n";
    lastError=E_UNREAD;
    return false;
  }

  // scan all files in the directory
  while ((dp=readdir(dirp)) != NULL) {
    if (strequ(dp->d_name,".") || strequ(dp->d_name,"..")) {
      continue;
    }

    buf=new char[strlen(dir)+strlen(dp->d_name)+2];
    if (buf==NULL) {
      cerr <<"Error allocating memory" << endl;
      lastError=E_MEMORY;
      return false;
    }
    sprintf(buf, "%s%c%s", dir, PATH_SEP, dp->d_name);

    stat(buf, &st);
    delete buf;

    // information are stored in directory names, not in files
    if (!S_ISDIR(st.st_mode)) {
      continue;
    }

    if (len=strlen(dp->d_name)==1) {
      tupdateVer=dp->d_name[0];
      present=true;
      TDEBUG("Found update version: " << tupdateVer  << endl);
    } else {
        delete hvscVer;
        hvscVer=new char[len+1];

        if (hvscVer==NULL) {
          cerr << "Error allocating memory" << endl;
          lastError=E_MEMORY;
          return false;
        }

        strcpy(hvscVer,dp->d_name);
        present=true;
        TDEBUG("Found HVSC version: " << hvscVer << endl);
      }
  }
  return present;
}

/**
 * Get the absolute path of file in tsid tree
 *
 * @param f the file name relative to tsid tree entry
 * @return an abs path name
 */
char *getAbs(char *f) {
  TDEBUG("Get absolute path of file: " << f << endl);

  char *p=new char[strlen(tsidDir)+strlen(f)+2];
  if (p==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return NULL;
  }

  sprintf(p, "%s%c%s", tsidDir, PATH_SEP, f);

  TDEBUG("Absolute= " << p << endl);

  return p;
}

/**
 * Init the TSID tree with the passed version
 *
 * @param hvscVer the hvsc version that the tsid tree represent
 * @param tupdateVer the version of the tupdate file previous installed
 * @return true if the initialization goes good
 */
bool initTreeVer(char *hvscVer, char tupdateVer) {
  TDEBUG("Init tree version with: " << hvscVer << " " << tupdateVer << endl);

  // check if this path exist, otherwise create it
  if (!tsid.checkDir(pathToDoc)) {
    lastError=E_CHECK;
    return false;
  }

  //be sure that the tree is not already initialized
  if (getTreeVer(pathToDoc)) {
    cout << "The TSID three is already initialized with: " <<
             ::hvscVer << " " << ::tupdateVer << endl;
    lastError=E_ALREADY;
    return false;
  }

  //initialize the tree
  char *tmp=new char[strlen(pathToDoc)+strlen(hvscVer)+2];
  if (tmp==NULL) {
      cerr << "Error allocating memory\n";
      lastError=E_MEMORY;
      return false;
  }
  sprintf(tmp, "%s%c%s", pathToDoc, PATH_SEP, hvscVer);

  if (!tsid.checkDir(tmp)) {
    lastError=E_CHECK;
    return false;
  }

  chmod(tmp, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);

  delete tmp;

  //initialize the tree
  tmp=new char[strlen(pathToDoc)+3];
  if (tmp==NULL) {
      cerr << "Error allocating memory\n";
      lastError=E_MEMORY;
      return false;
  }
  sprintf(tmp, "%s%c%c", pathToDoc, PATH_SEP, tupdateVer);

  if (!tsid.checkDir(tmp)) {
    lastError=E_CHECK;
    return false;
  }

  chmod(tmp, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);

  delete tmp;
  return true;
}

/**
 * ReInit the TSID tree with the passed version
 *
 * @param hvscVer the hvsc version that the tsid tree will be
 * @param tupdateVer the version of the tupdate file that will be
 * @return true if the initialization goes good
 */
bool reInitTreeVer(char *hvscVer, char tupdateVer) {
  dirent *dp;
  struct stat st;
  DIR    *dirp;
  int len;
  char *buf;

  TDEBUG("Reinit the TSID tree with: " << hvscVer << " " << tupdateVer << endl);

  //remove the previous version

  dirp=opendir(pathToDoc);

  if (dirp==NULL) {
    cerr << pathToDoc << " unreadable\n";
    lastError=E_UNREAD;
    return false;
  }

  // scan all files in the directory
  while ((dp=readdir(dirp)) != NULL) {
    if (strequ(dp->d_name,".") || strequ(dp->d_name,"..")) {
      continue;
    }

    buf=new char[strlen(pathToDoc)+strlen(dp->d_name)+2];
    if (buf==NULL) {
      cerr <<"Error allocating memory\n";
      lastError=E_MEMORY;
      return false;
    }
    sprintf(buf, "%s%c%s", pathToDoc, PATH_SEP, dp->d_name);

    stat(buf, &st);
    delete buf;

    // information are stored in directory names, not in files
    if (S_ISDIR(st.st_mode)) {
      TDEBUG("Remove: " << buf << endl);

      // remove the directory
      if (remove(buf)!=0) {
        cerr << "Error removing directory\n";
        lastError=E_REMOVE;
        return false;
      }
    }
  }

  //add the new version
  return initTreeVer(hvscVer, tupdateVer);
}

/**
 * Get parameters summary as an integer value
 *
 * @param argc the numbr of passed string
 * @param argv the strings of parameters
 * @return an integer that summary all the options available
 */
int getParameter(int argc, char** argv) {
  int value=P_NOTHING;                       // returned value;

  // parse command line arguments
  int a=1;
  while ((a<argc) && (argv[a] != NULL))	{
    if (argv[a][0]=='-') {
      int b=1;
      while (argv[a][b]!='\0') {
        switch (argv[a][b]) {
          case 'u':                          // update tree
            value|=P_UPDATE;
            break;
          case 'b':                          // backup tree
            value|=P_BACKUP;
            break;
          case 'h':                          // help
            value|=P_HELP;
            break;
          case 'i':                          // init tree
            value|=P_INIT;
            break;
          default:
            return P_ERROR;
        }
        b++;                                 // next char
      }
    } else {
        // option must be specify with -
        return P_ERROR;
      }

    a++;                                     // next parameter
  }
  return value;
}

/**
 * Backup the tsid tree with a tar.gz or zip file
 *
 * @return true if operation is ok
 */
bool backup() {
  char *tar="tar -czf tsid_backup.tar.gz ";
  char *tarCom=NULL;

  TDEBUG("Back up of tsid tree directory\n");

  tarCom=new char [strlen(tar)+strlen(tsidDir)+1];

  if (tarCom==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }

  sprintf(tarCom, "%s%s", tar, tsidDir);
  TDEBUG("Tar command to execute: " << tarCom << endl);

  // backup with tar/gzip if present
  if (system(tarCom)==0) {
    delete tarCom;
    return true;                         // backup is done
  };

  delete tarCom;

  //try with zip
  ///not yet implemented
  lastError=E_SYSTEM;
  return false;

  return true;
}

/**
 * Get the string before the slash of a passed string
 * ///taken from TSID. This is not a good solution, just a work around
 *
 * @param str the passed string
 * @return the expected string
 */
char *getStrBeforeSlash(char *str) {
  int length;                                // length of result string
  char *pnt=&str[strlen(str)];               // pointer to the end of string

  while (*pnt!=PATH_SEP) {                   // look for slash
    pnt--;                                   // decrement pointer
    if (pnt<=str) break;                     // avoid endless loop
  }

  char *result=new char[(length=(pnt-str))+1];
  if (result==NULL) {
      cerr << "Error allocating memory\n";
      lastError=E_MEMORY;                    // no memory available
      return NULL;
  }
  strncpy(result, str, length);

  result[length]='\0';                       // terminate the string

  WDEBUG("BeforeSlash: \""<< result << "\"\n");
  return result;
}

/**
 * Check if the directory of this file exist
 *
 * @param f the file name with path
 * @return true if ok
 */
bool check(char *f) {
  char *p;
  p=getStrBeforeSlash(f);
  tsid.checkDir(p);

  // check if this path exist, otherwise create it
  if (!tsid.checkDir(p)) {
    lastError=E_CHECK;
    return false;
  }
  delete p;
  return true;
}

/**
 * Update the TSID tree.
 */
bool update() {
  const int LEN=10;
  const int LENF=256;

  char *name;
  char instr[LEN]   = "";   // instruction name
  char file1[LENF]  = "";   // file name 1
  char file2[LENF]  = "";   // file name 2

  char *pf1;                // path and name of file 1
  char *pf2;                // path and name of file 2

  char oldHVSC[LEN] = "";   // old hvsc version
  char oldUpdate    = '\0'; // old update version

  char newHVSC[LEN] = "";   // new hvsc version
  char newUpdate    = '\0'; // new update version

  double version    = 0;    // version of the required library

#define FILE_GET(AB) if (file.get()!=AB) { \
                      cerr << "Error parsing update file\n"; \
                      lastError=E_SYNTAX; \
                      return false; \
                    }

  TDEBUG("Update the tsid tree\n");

  // get actual tsid tree version
  if (!getTreeVer(pathToDoc)) {
    cout << "Your TSID tree is not yet initialized. The initialization must " <<
            "be done before you update your tree. Use update -i\n";
    lastError=E_INIT;
    return false;
  }

  //make a backup copy of your tsid tree
  if (!backup()) {
    cerr << "I'm not able to create a backup copy of your TSID tree\n";
    return false;
  }

  //open the update file from home directory
  name=new char[strlen(homeDir)+strlen(updateFile)+2];
  if (name==NULL) {
    cerr <<"Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }
  sprintf(name, "%s%c%s", homeDir, PATH_SEP, updateFile);

  // open the file
  ifstream file(name, ios::in);

  if (!file) {
    cerr << "The update.thvs file is not present in your home directory\n";
    lastError=E_MISSING;
    return false;
  }

  // search for the expected VER x.x instruction
  file.get(instr, LEN, ' ');
  if (!(strequ(instr,"VER"))) {
    cerr << "Missing expected VER instruction from tvhs file\n";
    lastError=E_VER;
    return false;
  }

  FILE_GET(' ');
  file >> version;
  TDEBUG("VER " << version << endl);
  FILE_GET('\n');

  // search for the expected FROM x.x y instruction
  file.get(instr, LEN, ' ');
  if (!(strequ(instr,"FROM"))) {
    cerr << "Missing expected FROM instruction from tvhs file\n";
    lastError=E_FROM;
    return false;
  }

  // verify that the version is good
  if (version>(double)tsid_version) {
    cerr << "You must use a tupdate tool of library " << version <<
            " for using this file\n";
    lastError=E_VERSION;
    return false;
  }

  FILE_GET(' ');
  file.get(oldHVSC, LEN, ' ');
  FILE_GET(' ');
  oldUpdate=file.get();
  TDEBUG("FROM " << oldHVSC << " " << oldUpdate << endl);
  FILE_GET('\n');

  //search for the expected TO x.x y instructions
  file.get(instr, LEN, ' ');
  if (!(strequ(instr,"TO"))) {
    cerr << "Missing expected TO instruction from thvs file\n";
    lastError=E_TO;
    return false;
  }

  FILE_GET(' ');
  file.get(newHVSC, LEN, ' ');
  FILE_GET(' ');
  newUpdate=file.get();
  TDEBUG("TO " << newHVSC << " " << newUpdate << endl);
  //FILE_GET('\n');

  // verify that the versions are corrected
  if (!(strequ(hvscVer, oldHVSC))) {
    cerr << "Your version of the tree (" << hvscVer <<
            ") is different from the expected ("  << oldHVSC << ")" << endl;
    lastError=E_VERSION;
    return false;
  }
  if ((strequ(hvscVer, newHVSC)) && (tupdateVer=='@')) {
    cout << "You don't need this update.\n";
    lastError=E_OK;
    return true;
  }
  if ((oldUpdate!=tupdateVer) && (tupdateVer!='@')) {
    cerr << "This is not the right update for your tree\n";
    lastError=E_VERSION;
    return false;
  }

  // update the tsid version in the tree
  reInitTreeVer(newHVSC, newUpdate);

  char ch;
  int num;
  int song;
  int song2;

  while (file.get(ch)) {
    if (ch!='\n') {
      cerr << "Line not ended as expected\n";
      lastError=E_SYNTAX;
      return false;
    }

    file.get(instr, LEN, ' ');

    // is a MOVE instruction?
    if (strequ(instr, "MOVE")) {
      FILE_GET(' ');
      file.get(file1, LENF, ' ');
      FILE_GET(' ');
      file.get(file2, LENF, '\n');

      pf1=getAbs(file1);
      pf2=getAbs(file2);
      check(pf2);
      instMOVE(pf1, pf2);

      delete pf1;
      delete pf2;
    }

    // is a delete instruction?
    if (strequ(instr, "DELETE")) {
      FILE_GET(' ');
      file.get(file1, LENF, '\n');

      pf1=getAbs(file1);
      instDELETE(pf1);

      delete pf1;
    }

    // is a delete instruction?
    if (strequ(instr, "SHIFT")) {
      FILE_GET(' ');
      file.get(file1, LENF, ' ');
      FILE_GET(' ');

      file >> num;
      FILE_GET(' ');
      FILE_GET('#');
      file >>song;

      pf1=getAbs(file1);
      instSHIFT(pf1, num, song);

      delete pf1;
    }

    // is a GOES instruction?
    if (strequ(instr, "GOES")) {
      FILE_GET(' ');
      file.get(file1, LENF, ' ');
      FILE_GET(' ');
      file.get(file2, LENF, ' ');
      FILE_GET(' ');
      FILE_GET('#');

      file >> song;

      FILE_GET(' ');
      FILE_GET('#');

      file >> song2;

      pf1=getAbs(file1);
      pf2=getAbs(file2);
      check(pf2);
      instGOES(pf1, pf2, song, song2);

      delete pf1;
      delete pf2;
    }

    // is a SWAP instruction?
    if (strequ(instr, "SWAP")) {
      FILE_GET(' ');
      file.get(file1, LENF, ' ');
      FILE_GET(' ');
      FILE_GET('#');

      file >> song;
      FILE_GET(' ');
      FILE_GET('#');
      file >>song2;

      pf1=getAbs(file1);
      instSWAP(pf1, song, song2);

      delete pf1;
    }

    // is a comment?
    if (strequ(instr, "%")) {
      file.get(file1, LENF, '\n');
      TDEBUG("Comment: " << file1 << endl);
    }
  }

  delete name;
  return true;
}

/**
 * Show a help screen
 */
void showHelp() {
  cout << "Syntax: tupdate -<option>\n";
  cout << "Options:\n";
  cout << " -h          display this screen\n";
  cout << " -i          init tsid tree\n";
  cout << " -u          update tsid tree\n";
  cout << " -b          backup tsid tree\n";
  cout << endl;
}

/**
 * Get some directory of user
 * Global variables are chaged
 * Note: this code is taken and modified from XMame!!!
 *
 * @return true if operation is ok
 */
bool getDir() {
  struct passwd *pw;

  // get home directory of user

  pw=getpwuid(getuid());
  if(!pw) {
    cerr << "Who are you? Not found in passwd database!!\n";
    lastError=E_WHO;
    return false;
  }

  homeDir=new char[strlen(pw->pw_dir)+1];
  if (homeDir==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }
  strcpy(homeDir, pw->pw_dir);

  TDEBUG("Home directory is: " << homeDir << endl);

  // get tsid directory
  tsidDir=new char [strlen(homeDir)+strlen(TDir)+2];

  if (tsidDir==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }
  sprintf(tsidDir, "%s%c%s", homeDir, PATH_SEP, TDir);

  TDEBUG("Path to tsid tree is: " << tsidDir << endl);

  pathToDoc=new char [strlen(tsidDir)+strlen(TDoc)+2];

  if (pathToDoc==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }
  sprintf(pathToDoc, "%s%c%s", tsidDir, PATH_SEP, TDoc);

  TDEBUG("Path to documentation is: " << pathToDoc << endl);

  return true;
}


/**
 * Init the TSID tree if all are good
 *
 * @return true if operation is ok
 */
bool initTree() {
  // try to found a valid hvsc
  char *env=getenv("HVSC_DIR");
  if (env==NULL) {
    cerr << "No enviroment for HVSC_DIR set up\n\n";
    cerr << "You must set up the HVSC_DIR to the HVSC directory.\n";
    cerr << "Use the hvsctest program to set up it correctly\n";
    lastError=E_ENVIR;
    return false;
  }
  TDEBUG("Your HVSC_DIR is set up to " << env << endl);

  char *name=new char[strlen(env)+30];
  if (name==NULL) {
    cerr << "Error allocating memory\n";
    lastError=E_MEMORY;
    return false;
  }
  sprintf(name, "%s%c%s%c%s",env, PATH_SEP, "DOCUMENTS", PATH_SEP, "hv_sids.txt");

  // open the file
  ifstream file(name, ios::in);

  if (!file) {
    cerr << "No valid HVSC found: missing file\n";
    lastError=E_MISSING;
    return false;
  }

  char ch;

  while (file.get(ch)) {
    if (ch=='s') break;
  }

  double ver;

  file.get(ch);
  if (ch=='e') {

    file >> ver;
    cout << "Found HVSC release " << ver << endl;
  } else {
      cerr << "No valid hv_sidt.txt file found\n";
      lastError=E_NOHVSC;
      return false;
   }

  char *verS=new char[256];

  if (verS==NULL) {
    cerr << "Error allocating memory\n";
    lastError=false;
    return 2;
  }
  gcvt(ver, 3, verS);

  initTreeVer(verS, '@');
  return true;
}

int main(int argc, char** argv) {
  int result;                                // parameters results

  cout << "TUPDATE 1.00\n";
  cout << "This program update your TSID tree\n";

  // get parameters value
  result=getParameter(argc, argv);

  if ((result & P_ERROR) || (result==P_NOTHING)) {    // there's a error?
    cout << "Invalid or missing parameter option. Try 'examiner -h' for a syntax help\n";
    return E_PARAM;
  }

  if (result & P_HELP) {                     // help option?
    showHelp();
    return E_OK;
  }

  // get some user directories
  if (!getDir()) {
    return lastError;
  }

  if (result & P_INIT) {                     // init tree
    if (initTree()) {
      cout << "Your TSID tree is now initialized\n";
      return E_OK;
    } else return lastError;
  }

  if (result & P_BACKUP) {                   // make backup
    if (backup()) return E_OK;
    else return lastError;
  }

  if (result & P_UPDATE) {                   // make update
    if (update()) {
      cout << "Your TSID tree in now updated\n";
      return E_OK;
    } else return lastError;
  }

}

// _TUpdate.CPP
